<?php

class db
{
	
	var $db;
	var $mode = 0666;
	var $error;
	var $db_file;
	
	function openDB($db_file) {
		global $conf;
		if ($this->db = sqlite_open($db_file, $this->mode, $this->error)) {
			$this->db_file = $db_file;
     		return true;
		} else {
    		return $this->error;
		}		
	}
	
	function query($query_string) {
		$res = sqlite_query($this->db,$query_string);
		$error_num = sqlite_last_error($this->db);
		if($error_num > 0) $this->error = sqlite_error_string($error_num);
		return $res;
	}
	
	function queryOneRecord($sql, $result_type = SQLITE_ASSOC) {
		$res = $this->query($sql);
		return sqlite_fetch_array($res,$result_type);
	
	}
	
	function queryAllRecords($sql, $result_type = SQLITE_ASSOC) {
		return sqlite_array_query($this->db,$sql,$result_type);
	}
	
	
	function closeDB() {
		sqlite_close($this->db);
	}
	
	function insertID() {
		return sqlite_last_insert_rowid($this->db);
	}
	
	function make_demotable() {
	
		sqlite_query($this->db,'CREATE TABLE foo (bar varchar(10))');
    	sqlite_query($this->db,"INSERT INTO foo VALUES ('fnord')");
    	$result = sqlite_query($this->db,'select bar from foo');
    	var_dump(sqlite_fetch_array($result));
	
	}
	
	function esc($val) {
		return sqlite_escape_string($val);
	}
	
	function debug($data) {
		
		echo "<pre>";
		print_r($data);
		echo "</pre>";
		die();
		
	}
	
	function tableAddField($table,$name,$type,$length = '',$default = '',$notnull = 0,$autoinc = 0,$index = '') {
		
		// Checking Values
		
		if($autoinc == 1) {
			$default = 'null';
			$index = 'primary';
			$type = 'INTEGER';
		}
		
		if($type == "VARCHAR") {
			if(empty($length)) $length = 255;
			$type = $type."(".$length.")";
		}
		
		
		// Hole Tabellen Info
		$tbl_info = $this->queryAllRecords("PRAGMA table_info('$table')");
		
		// fge neue column hinzu
		$tbl_new = $tbl_info;
		
		$tbl_new[] = array(	cid 		=> 999,
    						name 		=> $name,
    						type 		=> $type,
    						notnull 	=> $notnull,
    						dflt_value 	=> $default);
		
		
		
		$this->query("BEGIN TRANSACTION");
		
		// creating temporary table
		$sql = "CREATE TEMPORARY TABLE ".$table."_backup(";
		foreach($tbl_new as $field) {
			$sql .= $field["name"].",";
		}
		$sql = substr($sql,0,-1);
		$sql .= ");";
		$this->query($sql);
		
		// copying data
		$sql = "INSERT INTO ".$table."_backup SELECT ";
		foreach($tbl_info as $field) {
			$sql .= $field["name"].",";
		}
		$sql .= " '$default' AS $name FROM $table;";
		$this->query($sql);
		
		// Deleting Table
		$sql = "DROP TABLE $table";
		$this->query($sql);
		
		// creating new table
		$sql = "CREATE TABLE ".$table." (";
		foreach($tbl_new as $field) {
			$sql .= $field["name"]." ".$field["type"];
			if($field["dflt_value"] != '') $sql .= " DEFAULT '".$field["dflt_value"]."'";
			$sql .=",";
		}
		$sql = substr($sql,0,-1);
		$sql .= ");";
		$this->query($sql);
		
		// copying the data back
		$sql = "INSERT INTO ".$table." SELECT ";
		foreach($tbl_new as $field) {
			$sql .= $field["name"].",";
		}
		$sql = substr($sql,0,-1);
		$sql .= " FROM ".$table."_backup;";
		$this->query($sql);
		
		// Deleting Backup Table
		$sql = "DROP TABLE ".$table."_backup";
		$this->query($sql);
		
		$this->query("COMMIT");
		
		// creating Index
		if($index != '') {
			$sql = "CREATE ";
			if($index == "unique") $sql .= "UNIQUE ";
			$sql .= "INDEX ".$table."_".$name."_idx ON ".$table." (".$name.")";
			$this->query($sql);
		}
		
	}
	
	function tableDeleteField($table,$name) {
		
		// Hole Tabellen Info
		$tbl_info = $this->queryAllRecords("PRAGMA table_info('$table')");
		
		// Delete column
		foreach($tbl_info as $col) {
			if($col["name"] != $name) $tbl_new[] = $col;
		}
		
		$this->query("BEGIN TRANSACTION");
		
		// creating temporary table
		$sql = "CREATE TEMPORARY TABLE ".$table."_backup(";
		foreach($tbl_new as $field) {
			$sql .= $field["name"].",";
		}
		$sql = substr($sql,0,-1);
		$sql .= ");";
		$this->query($sql);
		
		// copying data
		$sql = "INSERT INTO ".$table."_backup SELECT ";
		foreach($tbl_new as $field) {
			$sql .= $field["name"].",";
		}
		$sql = substr($sql,0,-1);
		$sql .= " FROM $table;";
		$this->query($sql);
		
		// Deleting Table
		$sql = "DROP TABLE $table";
		$this->query($sql);
		
		// creating new table
		$sql = "CREATE TABLE ".$table." (";
		foreach($tbl_new as $field) {
			$sql .= $field["name"]." ".$field["type"];
			if($field["dflt_value"] != '') $sql .= " DEFAULT '".$field["dflt_value"]."'";
			$sql .=",";
		}
		$sql = substr($sql,0,-1);
		$sql .= ");";
		$this->query($sql);
		
		// copying the data back
		$sql = "INSERT INTO ".$table." SELECT ";
		foreach($tbl_new as $field) {
			$sql .= $field["name"].",";
		}
		$sql = substr($sql,0,-1);
		$sql .= " FROM ".$table."_backup;";
		$this->query($sql);
		
		// Deleting Backup Table
		$sql = "DROP TABLE ".$table."_backup";
		$this->query($sql);
		
		$this->query("COMMIT");
		
	}
	
	function tableEditField($table,$name_orig,$name,$type,$length = '',$default = '',$notnull = 0) {
		
		// Hole Tabellen Info
		$tbl_info = $this->queryAllRecords("PRAGMA table_info('$table')");
		
		if($type == "VARCHAR") {
			if(empty($length)) $length = 255;
			$type = $type."(".$length.")";
		}
		
		// Delete column
		foreach($tbl_info as $col) {
			if($col["name"] == $name_orig) {
				
				$tbl_new[] = array(	cid => $col["cid"],
    						name 		=> $name,
    						type 		=> $type,
    						notnull 	=> $notnull,
    						dflt_value 	=> $default);
				
			} else {
				$tbl_new[] = $col;
			}
		}
		
		$this->query("BEGIN TRANSACTION");
		
		// creating temporary table
		$sql = "CREATE TEMPORARY TABLE ".$table."_backup(";
		foreach($tbl_new as $field) {
			$sql .= $field["name"].",";
		}
		$sql = substr($sql,0,-1);
		$sql .= ");";
		$this->query($sql);
		//echo $sql . "<br>";
		
		// copying data
		$sql = "INSERT INTO ".$table."_backup SELECT ";
		foreach($tbl_new as $field) {
			if($field["name"] == $name) {
				$sql .= $name_orig ." AS ". $field["name"].",";
			} else {
				$sql .= $field["name"].",";
			}
		}
		$sql = substr($sql,0,-1);
		$sql .= " FROM $table;";
		$this->query($sql);
		//echo $sql . "<br>";
		//die();
		
		// Deleting Table
		$sql = "DROP TABLE $table";
		$this->query($sql);
		
		// creating new table
		$sql = "CREATE TABLE ".$table." (";
		foreach($tbl_new as $field) {
			$sql .= $field["name"]." ".$field["type"];
			if($field["dflt_value"] != '') $sql .= " DEFAULT '".$field["dflt_value"]."'";
			$sql .=",";
		}
		$sql = substr($sql,0,-1);
		$sql .= ");";
		$this->query($sql);
		
		// copying the data back
		$sql = "INSERT INTO ".$table." SELECT ";
		foreach($tbl_new as $field) {
			$sql .= $field["name"].",";
		}
		$sql = substr($sql,0,-1);
		$sql .= " FROM ".$table."_backup;";
		$this->query($sql);
		
		// Deleting Backup Table
		$sql = "DROP TABLE ".$table."_backup";
		$this->query($sql);
		
		$this->query("COMMIT");
		
	}
	
	function databaseAddTable($table,$fields) {
		
		$table = trim($table);
		$this->error = '';
		
		// Checking Variables
		if($table == '') $this->error = "Table name is empty.";
		if(!is_array($fields)) $this->error = "Fields must be an array.";
		if($this->queryOneRecord("SELECT * FROM sqlite_master WHERE type='table' and name= '$table'")) $this->error = "Table $table already exists.";
		
		// Building query String
		$sql = "CREATE TABLE $table (";
		
		foreach($fields as $field) {
			
			// Changing values if autoinc is selected
			if($field["autoinc"] == 1) {
				//$field["default"] = 'null';
				//$field["notnull"] = 0;
				$field["index"] = 'primary';
				$field["type"] = 'INTEGER';
			}
			
			// type
			if($field["type"] == "VARCHAR") {
				$field["length"] = intval($field["length"]);
				if($field["length"] == 0) $field["length"] = 255;
				$type = $field["type"]."(".$field["length"].")";
			} else {
				$type = $field["type"];
			}
			
			$sql .= $field["name"]." ".$type;
			if($field["notnull"] == 1) $sql .= " NOT NULL";
			if($field["index"] == "primary") $sql .= " CONSTRAINT ".$field["name"]."_idx_primary PRIMARY KEY";
			if($field["index"] == "unique") $sql .= " CONSTRAINT ".$field["name"]."_idx_unique UNIQUE";
			if($field["default"] != '') $sql .= " DEFAULT '".$field["default"]."'";
			$sql .=",";
			
		}
		$sql = substr($sql,0,-1);
		$sql .= ")";
		
		if($this->error == '') {
			$this->query($sql);
		}
		
		// Building indices
		foreach($fields as $field) {
			if(!empty($field["index"])) {
				
				// check if index exists
				
				// build query
				$sql = "CREATE ";
				if($field["index"] == "unique") $sql .= "UNIQUE ";
				$sql .= "INDEX ".$table."_".$field["name"]."_idx ON ".$table." (".$field["name"].")";
				$this->query($sql);
				
			}
		}
	}
	
	function tableAddIndex($table, $name, $fields, $unique) {
		
		$sql = "CREATE ";
		if($unique == 1) $sql .= "UNIQUE ";
		$sql .= "INDEX ".$name." ON ".$table." (".$fields.")";
		$this->query($sql);
	}
	
	function tableDropIndex($name) {
		$this->query("DROP INDEX $name");
	}
	
}

?>